// take an array of raw data regarding the functions and return corresponding metrics
const computeFunctionMetrics = (rd) => {
  let numRegularFunctions = 0;
  let numArrowFunctions = 0;
  let totalNumParam = 0;
  let totalNumParamDefaultValues = 0;
  let totalLengthParamNames = 0;
  let numFunctionNames = 0;
  let totalLengthFunctionNames = 0;
  let totalNumLines = 0;
  let totalNumChara = 0;

  rd.forEach((e) => {
    // name related
    if (e.name) {
      numFunctionNames += 1;
      totalLengthFunctionNames += e.name.length;
    }

    // type related
    if (e.type === 'FunctionDeclaration') numRegularFunctions += 1;
    else if (e.type === 'ArrowFunctionExpression') numArrowFunctions += 1;

    // size related
    totalNumLines += e.numLines;
    totalNumChara += e.numChara;

    // parameter related
    if (e.params.length) {
      totalNumParam += e.params.length;
      totalLengthParamNames += e.params.reduce((t, p) => t + p.name.length, 0);
      totalNumParamDefaultValues += e.params.filter(p => p.defaultValue).length;
    }
  });

  return {
    numFunctions: rd.length,
    numRegularFunctions,
    numArrowFunctions,
    totalNumParam,
    totalNumParamDefaultValues,
    totalLengthParamNames,
    numFunctionNames,
    totalLengthFunctionNames,
    totalNumLines,
    totalNumChara,
  };
};

// takes an array of raw data about the files in the projet and generate the associated metrics
module.exports = rawData =>
  // TODO add check for array
  new Promise((resolve) => {
    const flatFunctions = rawData.map(e => e.functions).reduce((t, e) => t.concat(e), []);

    resolve({
      functions: computeFunctionMetrics(flatFunctions),
    });
  });
