const traverse = require('@babel/traverse');

// extract the number of lines from a babylon loc node
const getNumLinesFromLoc = l => l.end.line - l.start.line + 1;

// extract the number of chatacters from a babylon node
const getNumCharaFromNode = n => n.end - n.start;

// extract parameters raw data from related AST node
const extractParams = astNode =>
  astNode.map((e) => {
    if (e.type === 'AssignmentPattern') {
      return {
        name: e.left.name,
        defaultValue: true,
      };
    } else if (e.type === 'Identifier') {
      return {
        name: e.name,
        defaultValue: false,
      };
    }
    return {
      name: 'UNKNOWN TYPE OF PARAMETER',
      defaultValue: false,
    };
  });

// extract raw data related to functions from an AST
const extractFunctions = (ast) => {
  const functionsRawData = [];

  const visitor = {
    Function: {
      enter(p) {
        // get the name of the function if we can find one
        let name = null;
        if (p.type === 'FunctionDeclaration') {
          // 'regular' functions
          if (p.node.id) ({ name } = p.node.id);
        } else if (p.type === 'ArrowFunctionExpression') {
          // 'arrow' functions
          if (p.parent.type === 'VariableDeclarator') ({ name } = p.parent.id);
        }

        // add raw data on this function to the returned array
        functionsRawData.push({
          posLine: p.node.loc.start.line,
          name,
          type: p.type,
          numLines: getNumLinesFromLoc(p.node.loc),
          numChara: getNumCharaFromNode(p.node),
          params: extractParams(p.node.params),
        });
      },
    },
  };

  traverse.default(ast, visitor);
  return functionsRawData;
};

// extract general informations about the provided AST
// manually inject the path of the corresponding file
const extractFile = (ast, filePath) => ({
  numLines: getNumLinesFromLoc(ast.loc),
  numChara: getNumCharaFromNode(ast),
  filePath,
});

// takes an array of ASTs as input, returns array of corresponding raw data
module.exports = (asts, paths) =>
  // TODO add check for array
  new Promise((resolve) => {
    resolve(asts.map((e, i) => ({
      functions: extractFunctions(e),
      file: extractFile(e, paths[i]),
    })));
  });
